// (c) COPYRIGHT URI/MIT 1997-98
// Please read the full copyright statement in the file COPYRIGHT.  
//
// Authors: reza (Reza Nekovei)

// This is the main CGI program which extracts the requested variable from a
// FreeFrom data file. It generates the DDS in-memory from both the FreeForm
// format file and external file and uses it to locate the variable to process.
//
// ReZa 6/10/97

#include "config_ff.h"

static char rcsid[] not_used ={"$Id: ff_dods.cc,v 1.19 2001/09/28 23:19:43 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include <string>
#include <iostream>

#include "cgi_util.h"
#include "DDS.h"
#include "FFArray.h"
#include "FFGrid.h"
#include "DODSFilter.h"
#include "Error.h"
#include "ce_functions.h"

long BufPtr = 0; // cache pointer
long BufSiz =0; // Cache size
char *BufVal = NULL; // cache buffer
const string cgi_version = DODS_SERVER_VERSION;

extern void read_descriptors(DDS &das, const string &filename) throw(Error);


int 
main(int argc, char * argv[])
{
    DDS dds;
    dds.add_function("date", func_date);
    dds.add_function("date_range", func_date_range);
    dds.add_function("start_date", func_startdate);
    dds.add_function("end_date", func_enddate);
    dds.add_function("DODS_JDate", proj_dods_jdate);
    dds.add_function("DODS_Date", proj_dods_date);
    dds.add_function("DODS_StartDate", proj_dods_startdate);
    dds.add_function("DODS_EndDate", proj_dods_enddate);

    dds.add_function("time", func_time);
    dds.add_function("start_time", func_starttime);
    dds.add_function("end_time", func_endtime);
    dds.add_function("DODS_Time", proj_dods_time);
    dds.add_function("DODS_StartTime", proj_dods_starttime);
    dds.add_function("DODS_EndTime", proj_dods_endtime);

    dds.add_function("date_time", func_date_time);
    dds.add_function("start_date_time", func_startdate_time);
    dds.add_function("end_date_time", func_enddate_time);
    dds.add_function("DODS_Date_Time", proj_dods_date_time);
    dds.add_function("DODS_StartDate_Time", proj_dods_startdate_time);
    dds.add_function("DODS_EndDate_Time", proj_dods_enddate_time);

#if 0
    dds.add_function("decimal_year", func_decimal_year);
    dds.add_function("DODS_Decimal_Year", proj_dods_decimal_year);
#endif

    // Use date() and date_time() comparison functions.
    dds.add_function("DODS_Decimal_Year", proj_dods_decimal_year);
    dds.add_function("DODS_StartDecimal_Year", proj_dods_startdecimal_year);
    dds.add_function("DODS_EndDecimal_Year", proj_dods_enddecimal_year);

    DODSFilter df(argc, argv);
	
    try {
	if (!df.OK()) {
	    df.print_usage();
	    return 1;
	}
	
	df.set_cgi_version(cgi_version);

	if (df.version()) {
	  df.set_cgi_version(cgi_version);
	  df.send_version_info();
	  return 0;
	}

	// Set the filename. 12/22/98 jhrg
	dds.filename(df.get_dataset_name());

	read_descriptors(dds, df.get_dataset_name());

	df.read_ancillary_dds(dds);

	df.send_data(dds, stdout);
    }
    catch (Error &e) {
      ErrMsgT(e.get_error_message());
	set_mime_text(cout, dods_error, df.get_cgi_version());
	e.print(cout);
	return 1;
    }

    return 0;
}

// $Log: ff_dods.cc,v $
// Revision 1.19  2001/09/28 23:19:43  jimg
// Merged with 3.2.3.
//
// Revision 1.18.4.3  2001/09/17 06:41:29  reza
// Fixed error reporting bugs.
//
// Revision 1.18.4.2  2001/06/09 00:25:20  jimg
// Changed the way the DODSFilter is used to support the new version request
// which returns information about the server's version (name and number)
// along with information about the core software's version.
//
// Revision 1.18.4.1  2001/05/23 18:14:53  jimg
// Merged with changes on the release-3-1 branch. This apparently was not
// done corrrectly the first time around.
//
// Revision 1.18  2000/10/12 18:11:45  jimg
// Added more exception stuff. Reworked the get/read_description() functions
// so they throw Error and InternalErr instead of returning codes.
//
// Revision 1.17  2000/10/11 19:37:56  jimg
// Moved the CVS log entries to the end of files.
// Changed the definition of the read method to match the dap library.
// Added exception handling.
// Added exceptions to the read methods.
//
// Revision 1.16  2000/08/31 22:16:55  jimg
// Merged with 3.1.7
//
// Revision 1.15.2.1  2000/05/01 21:35:34  dan
// Added function declarations for new date-range server side functions.
//
// Revision 1.15  1999/07/22 20:54:27  jimg
// Resolved conflicts from the merge with the release-3-0-2 branch.
//
// Revision 1.14  1999/05/28 17:19:20  dan
// Added DODS_Decimal_Year server-side function definitions.
//
// Revision 1.13.2.2  1999/06/04 14:47:43  dan
// Changed synthesized variable name DODS_DYear back to DODS_Decimal_Year
//
// Revision 1.13.2.1  1999/05/31 17:36:40  jimg
// Added call for the DODS_DYear projection function. This means that the server
// knows what to do when users ask for the date to be returned using decimal
// years.
//
// Revision 1.13  1999/05/28 16:34:07  jimg
// Fixed comments.
//
// Revision 1.12  1999/05/27 17:02:23  jimg
// Merge with alpha-3-0-0
//
// Revision 1.11  1999/05/26 17:12:04  jimg
// Fixed the catch of Error &e so that the correct MIME header is sent with the
// serialized Error object.
//
// Revision 1.10.2.1  1999/05/20 21:43:42  edavis
// Fix spelling of COPYRIGHT and remove some #if 0 stuff.
//
// Revision 1.10  1999/05/04 02:55:38  jimg
// Merge with no-gnu
//
// Revision 1.9.8.1  1999/05/01 04:40:30  brent
// converted old String.h to the new std C++ <string> code
//
// Revision 1.9  1999/01/05 00:49:11  jimg
// Added ce functions for data_time. Removed some unnecessary ce functions (the
// selection functions for synthesized variables don't need to be registered).
//
// Revision 1.8  1998/12/22 22:30:17  jimg
// Added fix to set the filename of the dataset in the DDS.
//
// Revision 1.7  1998/11/10 03:47:22  jimg
// Add the new DODS_Date aand DODS_JDate projection functions. This is here to
// prototype the CS design.
//
// Revision 1.6  1998/09/17 17:42:12  jimg
// Added installation of date_func in DDS.
//
// Revision 1.5  1998/08/12 21:21:11  jimg
// Massive changes from Reza. Compatible with the new FFND library
//
// Revision 1.4  1998/04/21 17:14:08  jimg
// Fixes for warnings, etc
//
// Revision 1.3  1998/04/16 18:11:24  jimg
// Sequence support added by Reza
//
//
